use std::fs;

fn evaluate_expression(numbers: &[i64], operators: &[&str]) -> i64 {
    let mut result = numbers[0];
    for (i, &op) in operators.iter().enumerate() {
        match op {
            "+" => result += numbers[i + 1],
            "*" => result *= numbers[i + 1],
            "||" => {
                // Concatenate numbers as strings, then convert back to an integer
                let concatenated = format!("{}{}", result, numbers[i + 1]);
                result = concatenated.parse().expect("Invalid concatenation");
            }
            _ => panic!("Invalid operator"),
        }
    }
    result
}

fn can_match_target(target: i64, numbers: &[i64]) -> bool {
    let num_operators = numbers.len() - 1;
    let total_combinations = 3_usize.pow(num_operators as u32); // 3^num_operators
    let operators = ["+", "*", "||"];
    let mut current_ops = vec![""; num_operators];

    for combination in 0..total_combinations {
        let mut temp = combination;
        for i in 0..num_operators {
            current_ops[i] = operators[temp % 3];
            temp /= 3;
        }
        if evaluate_expression(numbers, &current_ops) == target {
            return true;
        }
    }

    false
}

fn total_calibration_result(input_data: &str) -> i64 {
    let mut total = 0;

    for line in input_data.lines() {
        if let Some((target_str, numbers_str)) = line.split_once(':') {
            let target: i64 = target_str.trim().parse().expect("Invalid target");
            let numbers: Vec<i64> = numbers_str
                .trim()
                .split_whitespace()
                .map(|num| num.parse().expect("Invalid number"))
                .collect();

            if can_match_target(target, &numbers) {
                total += target;
            }
        }
    }

    total
}

fn main() {
    let input_data = fs::read_to_string("input.txt").expect("Failed to read input.txt");
    let result = total_calibration_result(&input_data);
    println!("Total calibration result: {}", result);
}
